// NOTE: Shader automatically converted from Godot Engine 3.5.1.stable's SpatialMaterial.

shader_type spatial;
render_mode depth_draw_alpha_prepass,async_visible,blend_mix,depth_draw_opaque,cull_back,diffuse_burley,specular_schlick_ggx;
uniform vec4 albedo : hint_color;
uniform sampler2D texture_albedo : hint_albedo;
uniform float specular;
uniform float metallic;
uniform float roughness : hint_range(0,1);
uniform float point_size : hint_range(0,128);
uniform sampler2D texture_normal : hint_normal;
uniform float normal_scale : hint_range(-16,16);
uniform sampler2D texture_ambient_occlusion : hint_white;
uniform vec4 ao_texture_channel;
uniform float ao_light_affect;
uniform vec3 uv1_scale;
uniform vec3 uv1_offset;
uniform vec3 uv2_scale;
uniform vec3 uv2_offset;

uniform sampler2D texture_trail_positions : hint_albedo;
uniform sampler2D texture_trail_depth;
uniform float trail_position_row;
uniform float trail_width : hint_range(0.01, 10.0);

const float TEXEL_SIZE = 1.0 / 128.0;
const float HALF_TEXEL_SIZE = 0.5 * TEXEL_SIZE;

vec4 get_trail_data(int instance, float segment) {
	float pixel = 1.0 / 128.0;
	vec2 coord = vec2(
		(float(instance) * pixel),
		(trail_position_row * pixel) + (-segment * pixel)
		);
	vec4 data = texture(texture_trail_positions, coord + vec2(HALF_TEXEL_SIZE));
	return data;
}

const vec3 depth_flip = vec3(1.0, 1.0, 1.0);
const float depth_scale  = 0.1;

void vertex() {
	vec4 prev_trail_point = get_trail_data(INSTANCE_ID, VERTEX.z - 1.0);
	vec4 trail_point = get_trail_data(INSTANCE_ID, VERTEX.z);
	float thickness = trail_width;
	vec2 direction = normalize(trail_point.xy - prev_trail_point.xy);
	vec2 rotated = vec2(
		direction.y,
		-direction.x
	) * VERTEX.x * thickness;
	COLOR = vec4(1.0, 1.0, 1.0, min(trail_point.a, min(1.0, VERTEX.z - 1.0) - (VERTEX.z * TEXEL_SIZE)));
	NORMAL = vec3(0.0, 1.0, 0.0);
	BINORMAL = -vec3(1.0 - rotated.y, 0.0, rotated.y);
	TANGENT = cross(BINORMAL, NORMAL);
	UV = vec2(
		VERTEX.x,
		trail_point.b / thickness
	);
	VERTEX.xz = rotated + trail_point.xy;
}

void fragment() {
	vec2 base_uv = UV;
	vec4 albedo_tex = texture(texture_albedo,base_uv);
	ALBEDO = albedo.rgb * albedo_tex.rgb * COLOR.rgb;
	METALLIC = metallic;
	ROUGHNESS = roughness;
	SPECULAR = specular;
	NORMALMAP = texture(texture_normal,base_uv).rgb;
	NORMALMAP_DEPTH = normal_scale;
	AO = dot(texture(texture_ambient_occlusion,base_uv),ao_texture_channel);
	AO_LIGHT_AFFECT = ao_light_affect;
	ALPHA = COLOR.a;
}
